#ifndef HYBRD_CU_INCLUDED
#define HYBRD_CU_INCLUDED

#include "dogleg.cu"
#include "dpmpar.cu"
#include "enorm.cu"
#include "fdjac1.cu"
#include "qform.cu"
#include "qrfac.cu"
#include "r1mpyq.cu"
#include "r1updt.cu"
#include <hybrd.c>

#endif
